from __future__ import annotations

import jijmodeling as jm
from jijmodeling.expression.utils import get_order


def _set_needs_square(
    problem: jm.Problem, needs_square_dict: dict[str, bool] | None = None
) -> jm.Problem:
    """Set the auto_qubo attribute of the constraints in the problem.

    Args:
        problem (jm.Problem): problem
        needs_square_dict (dict[str, bool] | None, optional): A dictionary with constraint labels as keys and values to be assigned to the `auto_qubo` attributes. Defaults to None.
    """
    if needs_square_dict is None:
        needs_square_dict = {}

    if hasattr(problem, "constraints"):
        for label in needs_square_dict.keys():
            if label not in problem.constraints:
                raise KeyError(
                    f"constraint {label} does not exist in problem.constraints."
                )

        for label, const in problem.constraints.items():
            if label in needs_square_dict:
                const._auto_qubo = needs_square_dict[label]
            # When the constraint label is not added to the 'needs_square_dict', it defaults to `True` for linear constraints and `False` for non-linear constraints.
            else:
                order = max(
                    get_order(const.condition.left), get_order(const.condition.right)
                )
                const._auto_qubo = False if order >= 2 else True
        return problem
    # problme does not have constraints.
    else:
        return problem
